@@ -41,7 +41,8 @@ import platform
 from py_tools_ds.processing.progress_mon import ProgressBar
 
 from sicor.Tools.EnMAP.metadata import varsol
-from sicor.Tools.EnMAP.LUT import get_data_file, read_lut_enmap_formatted, interpol_lut_c, download_LUT
+from sicor.Tools.EnMAP.LUT import get_data_file, read_lut_enmap_formatted, download_LUT, interpol_lut_c
+from sicor.Tools.EnMAP.LUT import reduce_lut, interpol_lut_c_red
 from sicor.Tools.EnMAP.conversion import generate_filter, table_to_array
 from sicor.Tools.EnMAP.optimal_estimation import invert_function
 from sicor.Tools.EnMAP.multiprocessing import SharedNdarray, initializer, mp_progress_bar
@@ -477,14 +478,14 @@ class Fo(object):
         self.cpu = options["retrieval"]["cpu"]
         self.disable_progressbars = options["retrieval"]["disable_progressbars"]
         self.instrument = options["sensor"]["name"]
-        self.data_vnir = enmap_l1b.vnir.data
-        self.data_swir = enmap_l1b.swir.data
+        self.data_vnir = enmap_l1b.vnir.data[:].copy()
+        self.data_swir = enmap_l1b.swir.data[:].copy()
         self.resamp_alg = options["sensor"]["resamp_alg"]
         self.land_only = options["retrieval"]["land_only"]
+        self.water_mask_vnir = enmap_l1b.vnir.mask_landwater[:, :]
 
         if self.land_only:
             logger.info("SICOR is applied to land pixels only. This may result in edge effects, e.g., at coastlines...")
-            self.water_mask_vnir = enmap_l1b.vnir.mask_landwater[:, :]
             self.water_mask_swir = enmap_l1b.transform_vnir_to_swir_raster(array_vnirsensorgeo=self.water_mask_vnir,
                                                                            resamp_alg=self.resamp_alg,
                                                                            respect_keystone=False)
@@ -681,6 +682,23 @@ class Fo(object):
         self.lut2_vnir = lut2_all_res_vnir
         self.lut2_swir = lut2_all_res_swir
 
+        # reduce grid dimensionality of LUTs to increase interpolation speed
+        self.logger.info("Reducing grid dimensionality of LUT to increase interpolation speed...")
+        self.gp = [enmap_l1b.meta.geom_view_zenith, enmap_l1b.meta.geom_sun_zenith,
+                   options["retrieval"]["default_aot_value"], self.raa.mean()]
+
+        self.lut_red_fit, self.xnodes_red, self.nm_nodes_red, self.ndim_red, self.x_cell_red = reduce_lut(
+            lut1=self.lut1_fit, lut2=self.lut2_fit, xnodes=self.xnodes, nm_nodes=self.nm_nodes, ndim=self.ndim,
+            x_cell=self.x_cell, gp=self.gp, intp_wvl=self.wvl_sel)
+
+        self.lut_red_vnir, _, _, _, _ = reduce_lut(lut1=self.lut1_vnir, lut2=self.lut2_vnir, xnodes=self.xnodes,
+                                                   nm_nodes=self.nm_nodes, ndim=self.ndim, x_cell=self.x_cell,
+                                                   gp=self.gp, intp_wvl=self.wvl_vnir)
+
+        self.lut_red_swir, _, _, _, _ = reduce_lut(lut1=self.lut1_swir, lut2=self.lut2_swir, xnodes=self.xnodes,
+                                                   nm_nodes=self.nm_nodes, ndim=self.ndim, x_cell=self.x_cell,
+                                                   gp=self.gp, intp_wvl=self.wvl_swir)
+
         # calculate absorption coefficients of liquid water and ice
         warnings.filterwarnings("ignore")
         self.logger.info("Calculating absorption coefficients of liquid water and ice...")
@@ -713,6 +731,8 @@ class Fo(object):
         else:
             logger.info("Performing first guess water vapor retrieval based on a common band ratio using VNIR data...")
             self.cwv_fg = wv_band_ratio(data=self.data_vnir,
+                                        water_msk=self.water_mask_vnir,
+                                        land_only=self.land_only,
                                         fn_table=self.fn_table,
                                         vza=self.pt[:, :, 0].mean(),
                                         sza=self.pt[:, :, 1].mean(),
@@ -726,7 +746,7 @@ class Fo(object):
                                         idx=[73, 76, 81],
                                         disable=self.disable_progressbars)
 
-            self.cwv_fg[np.isnan(self.cwv_fg)] = self.prior_mean[0]
+            self.cwv_fg[np.logical_or(np.isnan(self.cwv_fg), np.isinf(self.cwv_fg))] = self.prior_mean[0]
 
         # transform CWV first guess map to SWIR sensor geometry to enable segmentation and 3 phases of water retrieval
         logger.info("Transforming CWV first guess map to SWIR sensor geometry to enable segmentation and 3 phases of "
@@ -864,15 +884,11 @@ class Fo(object):
         :return:        modeled TOA radiance
         """
         # LUT interpolation
-        vtest = np.asarray([pt[0], pt[1], pt[2], pt[3], pt[4], xx[0]])
-        f_int = interpol_lut_c(lut1=self.lut1_fit,
-                               lut2=self.lut2_fit,
-                               xnodes=self.xnodes,
-                               nm_nodes=self.nm_nodes,
-                               ndim=self.ndim,
-                               x_cell=self.x_cell,
-                               vtest=vtest,
-                               intp_wvl=self.wvl_sel)
+        vtest = np.asarray([pt[2], xx[0]])
+
+        f_int = interpol_lut_c_red(lut1=self.lut_red_fit[:, :, :, 0], lut2=self.lut_red_fit[:, :, :, 1:],
+                                   xnodes=self.xnodes_red, nm_nodes=self.nm_nodes_red, ndim=self.ndim_red,
+                                   x_cell=self.x_cell_red, vtest=vtest, intp_wvl=self.wvl_sel)
 
         f_int_l0 = f_int[0, :] * 1.e+3
         f_int_edir = f_int[1, :] * 1.e+3
@@ -904,35 +920,20 @@ class Fo(object):
         :return:     modeled surface reflectance
         """
         # LUT interpolation
-        vtest = np.asarray([pt[0], pt[1], pt[2], pt[3], pt[4], xx])
+        vtest = np.asarray([pt[2], xx])
 
         if mode == "vnir":
-            f_int = interpol_lut_c(lut1=self.lut1_vnir,
-                                   lut2=self.lut2_vnir,
-                                   xnodes=self.xnodes,
-                                   nm_nodes=self.nm_nodes,
-                                   ndim=self.ndim,
-                                   x_cell=self.x_cell,
-                                   vtest=vtest,
-                                   intp_wvl=self.wvl_vnir)
+            f_int = interpol_lut_c_red(lut1=self.lut_red_vnir[:, :, :, 0], lut2=self.lut_red_vnir[:, :, :, 1:],
+                                       xnodes=self.xnodes_red, nm_nodes=self.nm_nodes_red, ndim=self.ndim_red,
+                                       x_cell=self.x_cell_red, vtest=vtest, intp_wvl=self.wvl_vnir)
         elif mode == "swir":
-            f_int = interpol_lut_c(lut1=self.lut1_swir,
-                                   lut2=self.lut2_swir,
-                                   xnodes=self.xnodes,
-                                   nm_nodes=self.nm_nodes,
-                                   ndim=self.ndim,
-                                   x_cell=self.x_cell,
-                                   vtest=vtest,
-                                   intp_wvl=self.wvl_swir)
+            f_int = interpol_lut_c_red(lut1=self.lut_red_swir[:, :, :, 0], lut2=self.lut_red_swir[:, :, :, 1:],
+                                       xnodes=self.xnodes_red, nm_nodes=self.nm_nodes_red, ndim=self.ndim_red,
+                                       x_cell=self.x_cell_red, vtest=vtest, intp_wvl=self.wvl_swir)
         else:
-            f_int = interpol_lut_c(lut1=self.lut1_fit,
-                                   lut2=self.lut2_fit,
-                                   xnodes=self.xnodes,
-                                   nm_nodes=self.nm_nodes,
-                                   ndim=self.ndim,
-                                   x_cell=self.x_cell,
-                                   vtest=vtest,
-                                   intp_wvl=self.wvl_sel)
+            f_int = interpol_lut_c_red(lut1=self.lut_red_fit[:, :, :, 0], lut2=self.lut_red_fit[:, :, :, 1:],
+                                       xnodes=self.xnodes_red, nm_nodes=self.nm_nodes_red, ndim=self.ndim_red,
+                                       x_cell=self.x_cell_red, vtest=vtest, intp_wvl=self.wvl_sel)
 
         f_int_l0 = f_int[0, :] * 1.e+3 * self.fac
         f_int_edir = f_int[1, :] * 1.e+3 * self.fac
@@ -1126,7 +1127,9 @@ def __minimize__(fo, opt_func, unknowns=False, logger=None):
 
     if platform.system() == "Windows" and processes > 1:
         logger.warning('Multiprocessing is currently not available on Windows.')
-    if platform.system() == "Windows" or processes == 1:
+    if platform.system() == "Darwin" and processes > 1:
+        logger.warning('Multiprocessing is currently not available on macOS.')
+    if platform.system() == "Windows" or platform.system() == "Darwin" or processes == 1:
         logger.info("Singleprocessing on 1 cpu")
     else:
         logger.info("Setting up multiprocessing...")
@@ -1231,7 +1234,7 @@ def __minimize__(fo, opt_func, unknowns=False, logger=None):
     # check if operating system is 'Windows'; in that case, multiprocessing is currently not working
     # TODO: enable Windows compatibility for multiprocessing
 
-    if platform.system() == "Windows" or processes == 1:
+    if platform.system() == "Windows" or platform.system() == "Darwin" or processes == 1:
         initializer(globals(), globs)
         [mp_fun(ii) for ii in tqdm(rng, disable=fo.disable_progressbars)]
     else:
