@@ -27,6 +27,7 @@
 import numpy as np
 from multiprocessing import Pool
 from tqdm import tqdm
+import platform
 
 from sicor.Tools.EnMAP.multiprocessing import initializer
 
@@ -124,7 +125,7 @@ def empirical_line_solution(X, rdn_subset, data_l2a_seg, rows, cols, bands, segs
     else:
         locations_subset = np.zeros((segs, 2))
         for i in range(segs):
-            if np.count_nonzero(segs == i) == 0:
+            if np.count_nonzero(labels.flat == i) == 0:
                 pass
             else:
                 locations_subset[i, :] = locations[labels.flat == i, :].mean(axis=0)
@@ -144,7 +145,7 @@ def empirical_line_solution(X, rdn_subset, data_l2a_seg, rows, cols, bands, segs
     global _globs
     _globs = dict(tree=tree, locs=locations_subset, k=15, rdn=rdn_subset, data=data_l2a_seg, bands=bands)
 
-    if processes == 1:
+    if platform.system() == "Windows" or platform.system() == "Darwin" or processes == 1:
         initializer(globals(), _globs)
         results = []
         for ii in tqdm(unique_labels, disable=disable_progressbars):
